﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace SongVocalSectionAnalyser.Common.Collections
{
    public static class CollectionExtensionMethod
    {
        public static T[][] ChunkBy<T>(this IEnumerable<T> source, int chunkSize)
        {
            return source
                .Select((x, i) => new { Index = i, Value = x })
                .GroupBy(x => x.Index / chunkSize)
                .Select(x => x.Select(v => v.Value).ToArray())
                .ToArray();
        }

        [DebuggerDisplay("{" + nameof(Key) + "} [{Elements.Length}]")]
        public class Group<TKey, TElement>
        {
            private readonly List<TElement> _elements = new List<TElement>();
            public TElement[] Elements => _elements.ToArray();

            public Group(TKey key) => Key = key;

            public void Add(TElement element) => _elements.Add(element);

            public TKey Key { get; }
        }

        public static Group<TKey, TElement>[] GroupContiguous<TElement, TKey>(this IEnumerable<TElement> sequence, Func<TElement, TKey> keySelector) => GroupContiguous(sequence, keySelector, EqualityComparer<TKey>.Default);

        public static Group<TKey, TElement>[] GroupContiguous<TElement, TKey>(this IEnumerable<TElement> sequence, Func<TElement, TKey> keySelector, IEqualityComparer<TKey> comparer)
        {
            var result = new List<Group<TKey, TElement>>();
            Group<TKey, TElement> group = null;
            foreach (var element in sequence)
            {
                var key = keySelector(element);
                if (null == group || false == comparer.Equals(group.Key, key))
                {
                    group = new Group<TKey, TElement>(key);
                    result.Add(group);
                    group.Add(element);
                }
                else
                {
                    group.Add(element);
                }
            }
            return result.ToArray();
        }

    }
}
